import numpy as np
import sleap
from sleap.nn.evals import load_metrics, compute_oks


sleap.use_cpu_only()


def test_compute_oks():
    inst_gt = np.array([[0, 0], [1, 1], [2, 2]]).astype("float32")
    inst_pr = np.array([[0, 0], [1, 1], [2, 2]]).astype("float32")
    oks = compute_oks(inst_gt, inst_pr)
    np.testing.assert_allclose(oks, 1)

    inst_pr = np.array([[0, 0], [1, 1], [np.nan, np.nan]]).astype("float32")
    oks = compute_oks(inst_gt, inst_pr)
    np.testing.assert_allclose(oks, 2 / 3)

    inst_gt = np.array([[0, 0], [1, 1], [np.nan, np.nan]]).astype("float32")
    inst_pr = np.array([[0, 0], [1, 1], [2, 2]]).astype("float32")
    oks = compute_oks(inst_gt, inst_pr)
    np.testing.assert_allclose(oks, 1)

    inst_gt = np.array([[0, 0], [1, 1], [np.nan, np.nan]]).astype("float32")
    inst_pr = np.array([[0, 0], [1, 1], [np.nan, np.nan]]).astype("float32")
    oks = compute_oks(inst_gt, inst_pr)
    np.testing.assert_allclose(oks, 1)


def test_load_metrics(min_centered_instance_model_path):
    model_path = min_centered_instance_model_path

    metrics = load_metrics(f"{model_path}/metrics.val.npz")
    assert "oks_voc.mAP" in metrics

    metrics = load_metrics(model_path, split="val")
    assert "oks_voc.mAP" in metrics

    metrics = load_metrics(model_path, split="train")
    assert "oks_voc.mAP" in metrics
